using System;
using System.IO;
using Google.Protobuf;
using Device;
using Operator;
using Face;
using User;

namespace example
{
	class TestImageUser
	{
		private FaceSvc faceSvc;
    private UserSvc userSvc;

    private const string FACE_WARPED_IMAGE = "_warped.jpg";

		public TestImageUser(FaceSvc fSvc, UserSvc uSvc) {
			faceSvc = fSvc;
      userSvc = uSvc;
		}

    public void setImageData(string fileName, ref byte[] imagedata)
    {
      File.WriteAllBytes(fileName, imagedata);
    }

    public void getImageData(string fileName, out byte[] imageData)
    {
      imageData = File.ReadAllBytes(fileName);
    }

    public ByteString NormalizeImage(uint deviceID, string rawImageFileName)
    {
      Console.WriteLine("Normalize from {0}, with {1}", deviceID, rawImageFileName);

      byte[] imageData;
      getImageData(rawImageFileName, out imageData);

      int imageSize = Buffer.ByteLength(imageData);
      ByteString rawImageData = ByteString.CopyFrom(imageData);

      ByteString warpedImageData = faceSvc.Normalize(deviceID, rawImageData);

      Console.WriteLine("Normalize completed");

      const bool isWriteImage = true;
      if (isWriteImage)
      {
        byte[] data = warpedImageData.ToByteArray();
        string warpedFileName = rawImageFileName + FACE_WARPED_IMAGE;
        setImageData(warpedFileName, ref data);
      };

      return warpedImageData;
    }

    public void EnrollFaceUser(uint deviceID, ref ByteString warpedImageData)
    {
      Console.WriteLine("Enroll user");

      string userID = "10";
      UserInfo userInfo = new UserInfo{Hdr = new UserHdr{ID = userID}};
      userInfo.Name = "testCsharpUser";
      userInfo.Setting = new UserSetting{StartTime = 978307200, EndTime = 1924991999};

      const uint Bs2FaceFlagF2Warped = 0x1;
      FaceData faceData = new FaceData();
      faceData.Flag = (uint)Face.FaceFlag.Bs2FaceFlagF2 | Bs2FaceFlagF2Warped;
      faceData.ImageData = warpedImageData;
      userInfo.Faces.Add(faceData);

      userSvc.Enroll(deviceID, new UserInfo[]{userInfo});

      Console.WriteLine("Enroll user finished");
    }
	}
}

